"""
This file holds various configuration options used for all of the examples.
"""
import os
import sys
# Use the colormath directory included in the downloaded package instead of
# any globally installed versions.
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))